// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <cstdio>
#include "LP.h"



void LP::solve(void){
  switch (lp_lib){
  case LRSLIB:
    solve_with_lrs();
    break;
  default:
    fatal("unknown or unimplemented solver");
  }
}

void LP::init_solver_lrs(void){
  
  if ( !lrs_init ((char*)("\n*LP")))
    fatal("initializing lrslib");

  if ( !solver_verbose ){
    FILE *bit_bucket=fopen("/dev/null","w");
    lrs_ofp=bit_bucket;
  }


  lplib_initialized=1;

}
void LP::solve_with_lrs(void){
  if (!lplib_initialized)
    init_solver_lrs();


  if (!(lrs_data=lrs_alloc_dat((char *)"LP globals")))
    fatal("lrs dat allocation");


  lrs_data->debug=solver_debug;

  lrs_data->minimize=1;


  lrs_data->m=(A.rows()+B.rows());

  lrs_data->n=A.cols();

  lrs_dict=lrs_alloc_dic(lrs_data);

  // set objective function
  set_lrs_row(0,-1*c,1);

  //  std::cerr << A;
  for (int i=0; i<A.rows(); i++){
    set_lrs_row(i+1,A[i],0);
  }
  //  std::cerr << B;
  for (int i=0; i<B.rows(); i++){
    set_lrs_row(i+A.rows()+1,B[i],1);
  }
  
  
  if (!lrs_solve_lp(lrs_dict,lrs_data))
    fatal("lrs solve");

  obj_val= Rational(mpz_class(lrs_dict->objnum),mpz_class(lrs_dict->objden));
  solution.clear();

  mpz_t *output;


  output= lrs_alloc_mp_vector (lrs_data->n);

    
  for (int col = 0; col < lrs_data->n; col++){
     if (lrs_getsolution (lrs_dict, lrs_data, output, col))
       for (int j=0; j<lrs_data->n; j++){
	 Rational r;
	 if (output[0]==0){
	   r=Rational(mpz_class(output[j]));
	 } else {
	   r=Rational(mpz_class(output[j]),mpz_class(output[0]));
	   r.canonicalize();
	 }
	 
	 solution.append(r);
       }
  }
  lrs_clear_mp_vector(output,lrs_data->n);
  lrs_free_dic(lrs_dict,lrs_data);
  lrs_free_dat(lrs_data);
  lrs_dict=0;
  lrs_data=0;
}

void LP::set_lrs_row(int row,const QVector& r, bool as_ineq){
  
  mpz_t denom[r.size()];
  mpz_t numer[r.size()];

  for (size_t j=0; j< r.size(); j++){
    mpz_init_set(denom[j],r[j].get_den_mpz_t());
    mpz_init_set(numer[j],r[j].get_num_mpz_t());
  }
  
  lrs_set_row_mp(lrs_dict,lrs_data,row,numer,denom,as_ineq);

  for (size_t j=0; j< r.size(); j++){
    mpz_clear(denom[j]);
    mpz_clear(numer[j]);
  }
}




void LP::fatal(const char *msg){
  std::cerr << "LP: " << msg << std::endl;
  exit(1);
}
