// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.
#ifndef _DB_PARTITION_H
#define _DB_PARTITION_H
#include <iostream>
#include <vector>
#include <set>
// iterate over partitions of 0..n-1, upto inversion



typedef  std::vector<unsigned int> ivec;
typedef  std::set<unsigned int> iset;
typedef ivec::const_iterator primary_iterator;
typedef iset::const_iterator complement_iterator;
class Partition {

 private:
  unsigned int cardinality;
  iset complement;
  ivec primary;

  void add_back(int i){
    primary.push_back(i);
    complement.erase(i);
  }
  void remove_back(){
    int t=primary.back();
    complement.insert(t);
    primary.pop_back();
  }
      
  void increment_back(){
    int newmax=primary.back();
    primary.pop_back();
    complement.insert(newmax);
    complement.erase(newmax+1);
    primary.push_back(newmax+1);
  }

 public:

  Partition(int n);
  void first();
  bool next();
  bool last();
  int card(){ return cardinality; };
  primary_iterator begin() const { return primary.begin(); };
  primary_iterator end() const { return primary.end(); };
  complement_iterator cbegin() const { return complement.begin(); };
  complement_iterator cend() const { return complement.end(); };

};


std::ostream& operator<< (std::ostream& os, const Partition & p);

#endif
