// David Bremner, 2005/08/27

// Enumerate affinely independant subsets $T$ of [-1,1]^d cube
// vertices, up to (part of?) the symmetry group of the cube.  It is
// bit easier to write down for the [0,1], cube, but dimension is
// obviously invariant under doubling the cube.

// So, think [0,1]^d for a minute
// The idea is that we may assume that
// 0) The  origin is in T.
// 1) At most one pair of inner diagonal vertices (i.e. negated) is in T.
// 2) W.l.o.g., if such a pair exists, it is (0,0,0,0..0), (1,1,...1)

// 3) By row/column permutation (which preserves the possible inner
// diagonal), we can assume the matrix T (with // points as rows) is
// in upper triangular form.

// 4) By the assumption the origin is the last row, lower
// triangular, and full rank, we know the last element of the every
// row but the last first is one. So actually we are looking at an upper
// triangular matrix of d-1 by k-1.

// 5) For a an triangular 0/1 matrix, with d-columns and k-rows,
// we have full rank iff we have enough distinct rows. Since we have the origin// we need to choose $k-1$ distinct positions for a leading one.
// This gives us binomial(d,k-1) possibilities, typically much smaller than  
// the binomial(d+k-1,k-1) monotone paths (upper triangular matrices).

#include "Symplex.hpp"

void Symplex::updateMatrix(){
  for (int i=0; i<rows-1; i++){
    for (int j=0; j<state[i]-1; j++)
      mat[i][j]=-1;
    for (int j=state[i]-1; j< dim; j++)
      mat[i][j]=1;
  }

  for (int j=0; j<dim; j++){
    mat[rows-1][j]=-1;
  }

  std::cerr << mat;
}

bool Symplex::next(){
  
  if (!state.next())
    return false;
  
  updateMatrix();
  return true;
}


